// this is an attempt to create a general-purpose abstract claas for o game of Go.
// The first such attempt was "Shobu.java", but it is a failure:
// it doesn't contain a general method for nextMove(), etc...

package Go;

import java.util.Vector;

public abstract class GoGame implements GoConstants {

  public GoGame() {
    listeners = new Vector();
  }

  public GoGame(int boardSize,
                Player blackPlayer, Player whitePlayer) {
    super();

    this.boardSize = boardSize;
    turn = BLACK;

    listeners = new Vector();

    goPosition = new GoPosition(boardSize);
    goPosition.setLastMove(new Move(GoConstants.WHITE));

    changedLocations = new ListOfStones();
    currentMove = new Move();


    this.blackPlayer = blackPlayer;
    this.blackPlayer.setGame(this);
    this.whitePlayer = whitePlayer;
    this.whitePlayer.setGame(this);

    // regiter the 2 players as game listeners
    addListener(blackPlayer);
    addListener(whitePlayer);

    gameResult = GoConstants.InProgress;
    nrCapturedStones = 0;
    consecutivePasses = 0;
  }

  // the main interface method: it performs the setup and starts the game
  public void playGame() {
    newGameSetup();

    while (!gameOver())
      playNextMove();
  }

  public void newGameSetup(){
    emptyGoban();
    turn = BLACK;
    gameResult = GoConstants.InProgress;

    blackPlayer.initPlayer();
    whitePlayer.initPlayer();

    oldGoPosition = null;
    goPosition = new GoPosition(boardSize);
  }

  // this method returns only after a new move was played
  public void playNextMove() {
    Player currentPlayer;

    if (turn == BLACK) currentPlayer = blackPlayer;
    else currentPlayer = whitePlayer;

    currentPlayer.makeMove(goPosition);

    /* No need for the next statement: it has been incorporated
       into Player.makeMove(); (the one from above)
    newMove(currentMove.x, currentMove.y); */
  }

  public abstract boolean gameOver();

  public int getGameResult() {
    if (!gameOver()) gameResult = GoConstants.InProgress;
    return gameResult;
  }

  public void switchColors() {
    Player aux;
    aux = blackPlayer;
    blackPlayer = whitePlayer;
    whitePlayer = aux;

    blackPlayer.color = BLACK;
    whitePlayer.color = WHITE;
  }

  protected void switchTurn() {
    if (turn == BLACK) turn = WHITE;
    else turn = BLACK;
  }

  // this method is validating a position resulted my the last move
  protected int validatePosition(GoPosition position) {

    return 1; // valid position
  }

  private void emptyGoban() {
    nrCapturedStones = 0;
    consecutivePasses = 0;

    for (int i = 0; i < boardSize; i++)
      for (int k = 0; k < boardSize; k++) {
        goPosition.goban[i][k].state = GoConstants.EMPTY;
        // in order to refresh the "liberties" list on game replay
        goPosition.goban[i][k].computeLiberties();
      }
  }

  // x, y are the coordinates of the new move
  // this returns > 0 if the move is valid and 0 otherwise
  public int newMove(int x, int y) {
    // todo: use the validate method (with a temp GoPosition)

    // check if the move was a Pass
    if ((x == GoConstants.PASS) || (y == GoConstants.PASS)) {
      // one more consecutive pass
      consecutivePasses++;
      switchTurn();

      return 1; // Pass move
    }
    else {
      nrCapturedStones = 0;
      consecutivePasses = 0;

      GobanLocation  stone;

      ListOfStones changedLocations = new ListOfStones();

      try {
        oldGoPosition = (GoPosition)goPosition.cloneCopy();
      } catch (CloneNotSupportedException e) {
          System.out.println(e);
        }

      stone = goPosition.addNewStone(x, y, turn);

      stone.computeLiberties();

      changedLocations.deleteAllElements();
      changedLocations.add(stone);

      goPosition.setNewMove(stone, changedLocations);

      nrCapturedStones = changedLocations.length() - 1;
      switchTurn();

      notifyPositionChanged(new GoGameEvent(this, oldGoPosition, goPosition));

      return 1;
    }
  }

  public void addListener(GoGameListener goGameListener) {
    if (!listeners.contains(goGameListener))
      listeners.addElement(goGameListener);
  }

  public void removeListener(GoGameListener goGameListener) {
    listeners.removeElement(goGameListener);
  }

  private synchronized void notifyPositionChanged(GoGameEvent goGameEvent) {
    // todo: make this thread-safe!!!

    java.util.Enumeration e = listeners.elements();
    GoGameListener goGameListener;

    while (e.hasMoreElements()) {
      goGameListener = (GoGameListener)e.nextElement();
      goGameListener.positionChanged(goGameEvent);
    }
  }

  private GoPosition oldGoPosition = null;
  public GoPosition goPosition = null;

  public int boardSize;

  protected int turn = BLACK;

  // the two players
  protected Player blackPlayer = null;
  protected Player whitePlayer = null;

  protected ListOfStones changedLocations = null;
  protected Move currentMove;

  protected int gameResult = GoConstants.InProgress;
  protected int nrCapturedStones = 0;
  protected int consecutivePasses = 0;

  private Vector listeners = null;
}
