package Go.strategy;

import Go.GoPosition;
import Go.Group;
import Go.GoConstants;
import Go.ListOfGroups;

import java.util.Enumeration; 

public class AtariGoEstimatorCho extends BasicGoEstimator {

  public AtariGoEstimatorCho() {
    super();
  }

  // BasicGoEstimator method implementation
  // Returns the state of the game in curent <position>, i.e:
  // BlackWIN, WhiteWIN, JIGO or InProgress
  public int checkPositionState(GoPosition position) {
    // here is the update: since we use info about captured stones,
    // it's much easier
    if ((position.getNrBlackPrisoners() == 0) && (position.getNrWhitePrisoners() == 0))
      return GoConstants.InProgress;
    else if ((position.getNrBlackPrisoners() > 0) && (position.getNrWhitePrisoners() == 0))
      return GoConstants.WhiteWIN;
    else if ((position.getNrBlackPrisoners() == 0) && (position.getNrWhitePrisoners() > 0))
      return GoConstants.BlackWIN;
    else {
      System.out.println("WRONG: checkPositionState: both black and white stones captured!");
      System.exit(1);
      return -10001;
    }
  }

  // BasicGoEstimator method implementation
  // returns a heuristic evaluation of the <position> from the point of view
  // of the <color> player
  protected float heuristicPositionEstimation(GoPosition position,
                                                       int color) {
    //return balancedPositionEstimation(position, color);
    //return aggresiveMinPositionEstimation(position, color);
    return aggresiveGlobalPositionEstimation(position, color);
  }

  // we'll estimate the position's value as own's group minimum nr liberties minus
  // the opponent's
  protected float balancedPositionEstimation(GoPosition position,
                                                       int color) {
    int myMinimum, oppMinimum, myCurrent, oppCurrent;
    ListOfGroups myGroups = null, oppGroups = null;

    if (color == GoConstants.BLACK) {
      myGroups = position.getBlackGroups();
      oppGroups = position.getWhiteGroups();
    }
    else if (color == GoConstants.WHITE) {
      myGroups = position.getWhiteGroups();
      oppGroups = position.getBlackGroups();
    }
    else {
      System.out.println("Unknown color: " + color);
      System.exit(1);
    }

    // compute my minimum
    myMinimum = Integer.MAX_VALUE;
    Enumeration myEnum = myGroups.getElements();
    for (;myEnum.hasMoreElements();) {
      myCurrent = ((Group)myEnum.nextElement()).nrLiberties();
      if (myCurrent < myMinimum)
        myMinimum = myCurrent;
    }

    // compute opp(onent's) minimum
    oppMinimum = Integer.MAX_VALUE;
    Enumeration oppEnum = oppGroups.getElements();
    for (;oppEnum.hasMoreElements();) {
      oppCurrent = ((Group)oppEnum.nextElement()).nrLiberties();
      if (oppCurrent < oppMinimum)
        oppMinimum = oppCurrent;
    }

    return myMinimum - oppMinimum;
  }

  // we'll estimate the position's value based on minimum nr liberties of opponent's groups
  protected float aggresiveMinPositionEstimation(GoPosition position,
                                                       int color) {
    int oppMinimum, oppCurrent;
    ListOfGroups oppGroups = null;

    if (color == GoConstants.BLACK) {
      oppGroups = position.getWhiteGroups();
    }
    else if (color == GoConstants.WHITE) {
      oppGroups = position.getBlackGroups();
    }
    else {
      System.out.println("Unknown color: " + color);
      System.exit(1);
    }

    // compute opp(onent's) minimum
    oppMinimum = Integer.MAX_VALUE;
    Enumeration oppEnum = oppGroups.getElements();
    for (;oppEnum.hasMoreElements();) {
      oppCurrent = ((Group)oppEnum.nextElement()).nrLiberties();
      if (oppCurrent < oppMinimum)
        oppMinimum = oppCurrent;
    }

    return (float)1 / oppMinimum;
  }

  // we'll estimate the position's value based on minimum nr liberties for own's groups
  protected float defensiveMinPositionEstimation(GoPosition position,
                                                 int color) {
    int ownMinimum, ownCurrent;
    ListOfGroups ownGroups = null;

    if (color == GoConstants.BLACK) {
      ownGroups = position.getBlackGroups();
    }
    else if (color == GoConstants.WHITE) {
      ownGroups = position.getWhiteGroups();
    }
    else {
      System.out.println("Unknown color: " + color);
      System.exit(1);
    }

    // compute opp(onent's) minimum
    ownMinimum = Integer.MAX_VALUE;
    
    Enumeration oppEnum = ownGroups.getElements();
    for (;oppEnum.hasMoreElements();) {
      ownCurrent = ((Group)oppEnum.nextElement()).nrLiberties();
      if (ownCurrent < ownMinimum)
        ownMinimum = ownCurrent;
    }

    return ownMinimum;
  }

  // we'll estimate the position's value based on global nr liberties of opponent's groups
  protected float aggresiveGlobalPositionEstimation(GoPosition position,
                                                    int color) {
    ListOfGroups oppGroups = null;

    if (color == GoConstants.BLACK) {
      oppGroups = position.getWhiteGroups();
    }
    else if (color == GoConstants.WHITE) {
      oppGroups = position.getBlackGroups();
    }
    else {
      System.out.println("Unknown color: " + color);
      System.exit(1);
    }

    // compute opp(onent's) total nr. liberties
    int result = 0;

    Enumeration oppEnum = oppGroups.getElements();
    for (;oppEnum.hasMoreElements();) {
      result += ((Group)oppEnum.nextElement()).nrLiberties();
    }

    if (result == 0)
      return 0;
    else
      return  (float) 1 / result;
  }
}
