package Go.strategy;

import Go.GoPosition;
import Go.GoConstants;

public abstract class BasicGoEstimator implements GoEstimatorInterface{

  // GoEstimatorInterface method implementation
  // this returns an estimation of the position's value, from
  // the point of view of the "color" player
  public float estimatePosition(GoPosition position, int color) {
    int gameState;

    // the game is still in progress
    if ((gameState = checkPositionState(position)) == GoConstants.InProgress)
      return heuristicPositionEstimation(position, color);

    // the game is over
    else
      return gameResultValue(gameState, color);
  }

  // returns a heuristic evaluation of the <position> from the point of view
  // of the <color> player
  protected abstract float heuristicPositionEstimation(GoPosition position,
                                                       int color);

  // returns the value of a final position, depending on the result
  // and "our" color
  protected float gameResultValue(int gameState, int color) {
    switch (gameState) {
    case GoConstants.BlackWIN:
      if (color == GoConstants.BLACK)
        return GoStrategyConstants.VictoryValue;
      else
        return GoStrategyConstants.LossValue;
    case GoConstants.WhiteWIN:
      if (color == GoConstants.BLACK)
        return GoStrategyConstants.LossValue;
      else
        return GoStrategyConstants.VictoryValue;
    case GoConstants.JIGO:
      return GoStrategyConstants.JigoValue;
    default:
      return GoStrategyConstants.UndefinedValue;
    }
  }
}

